#ifndef f_INPUTFILEMKV_H
#define f_INPUTFILEMKV_H

///////////////////////////////////////////////////////////////////////
//
//						InputFileMKV
//
///////////////////////////////////////////////////////////////////////

typedef struct {
	__int64       pos;
	unsigned long size;
	unsigned long pres_time;
} MKVSampleInfo;


class InputFileMKV : public vdxunknown<IVDXInputFile> {

private:
	MkvFile mFile;

	wchar_t*    MuxingApp;
	wchar_t*    WritingApp;

	HWND    hwndProgress;
	bool    fAbort;

	enum    { MAX_TRACKS = 32 };

	class   MatroskaTrack *Track[MAX_TRACKS];
	int     iTracks;

	int     iAudioTracks;
	int     iVideoTracks;

	__int64 iTimecodeScale;

	wchar_t *ReplaceExtension(const wchar_t *szFile, const wchar_t *szExt);
	wchar_t *GetFileNameFromIndex(const wchar_t *szIndex);
	wchar_t *GetIndexName(const wchar_t *szFile);
	bool    ReadIndex(const wchar_t *szFile);
	bool    WriteIndex(const wchar_t *szFile);

	bool    ParseBlockGroup(__int64 limit, const __int64 timebase);
	bool    ParseCluster(__int64 limit);
	bool    ParseTrackEntry(__int64 limit);
	bool    ParseTracks(__int64 limit);
	void    ParseInfo(__int64 limit);
	bool    ParseSegment(__int64 limit);
	bool    ParseHeader(__int64 limit);
	bool    ParseRoot();

	static  void SetDlgUserLong(HWND hDlg, LONG_PTR lParam);
	static  LONG_PTR GetDlgUserLong(HWND hDlg);

	static  char *time_to_string(double seconds);
	static  void UpdateDlgAudio(HWND hDlg, InputFileMKV *thisPtr, int index);
	static  inline void UpdateDlgVideo(HWND hDlg, InputFileMKV *thisPtr, int index);
	static  void SetItemTextW(HWND hDlg, int iItem, wchar_t *pText);
	static  INT_PTR CALLBACK InfoDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

	bool    guiDlgMessageLoop(HWND hDlg);

	static  BOOL CALLBACK FindContextWindow(HWND hwnd, LPARAM lParam);
	static  INT_PTR CALLBACK ParseDialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

	const wchar_t *GetFileTitle(const wchar_t *szFile);

protected:
	const   VDXInputDriverContext& mContext;

public:
	InputFileMKV(const VDXInputDriverContext& context);
	~InputFileMKV();

	// IVDXInputFile
	void VDXAPIENTRY Init(const wchar_t *szFile, IVDXInputOptions *opts);
	bool VDXAPIENTRY Append(const wchar_t *szFile);

	bool VDXAPIENTRY PromptForOptions(VDXHWND, IVDXInputOptions **);
	bool VDXAPIENTRY CreateOptions(const void *buf, uint32 len, IVDXInputOptions **);
	void VDXAPIENTRY DisplayInfo(VDXHWND hwndParent);

	bool VDXAPIENTRY GetVideoSource(int index, IVDXVideoSource **);
	bool VDXAPIENTRY GetAudioSource(int index, IVDXAudioSource **);

	// InputFileMKV
	static  int  lang_to_id(const char *lang);
	static  bool IsPCM(const MatroskaTrack &mt);
	int     ReadData(void *lpBuffer, __int64 offset, int bytes);
};

#endif	// f_INPUTFILEMKV_H
