///////////////////////////////////////////////////////////////////////
// Matroska plugin for VirtualDub 1.8.8+
// Copyright (C) 2010-2012 fccHandler
// Copyright (C) 1998-2012 Avery Lee
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///////////////////////////////////////////////////////////////////////

// IVDXInputOptions implementation

#if defined(_MSC_VER) && (_MSC_VER >= 1400)
	#define _CRT_SECURE_NO_WARNINGS
#endif

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>

#include "VDUnknown.h"
#include "VDInputDriver.h"
#include "InputOptionsMKV.h"
#include "Resource.h"

extern "C" const VDXPluginInfo *const * VDXAPIENTRY VDGetPluginInfo();

static const char MkvRegKey[] = "Software\\fccHandler\\Plugins\\Matroska";

extern HMODULE g_hModule;	// InputFileMKV.cpp


InputOptionsMKV::InputOptionsMKV()
{
	HKEY hKey;

	opts.len = sizeof(opts);

	// Defaults
	opts.fCreateIndex    = false;
	opts.fDeliverRawAAC  = true;
	opts.fUseH264NotAVC1 = false;

	if (RegOpenKeyEx(HKEY_CURRENT_USER, MkvRegKey,
		0, KEY_READ, &hKey) == ERROR_SUCCESS)
	{
		DWORD lpcbData = opts.len;
		RegQueryValueEx(hKey, "Options", NULL, NULL, (LPBYTE)&opts, &lpcbData);
		RegCloseKey(hKey);
	}
}


InputOptionsMKV::~InputOptionsMKV()
{
}


uint32 VDXAPIENTRY InputOptionsMKV::Write(void *buf, uint32 buflen)
{
	if (buf != NULL && buflen >= opts.len)
	{
		memcpy(buf, &opts, opts.len);
	}
	return opts.len;
}


void InputOptionsMKV::InitDialogTitle(HWND hDlg, char *pTitle)
{
	char buf[64];
	int i;

	uint32 ver = (*VDGetPluginInfo())->mVersion;

	i = _snprintf(buf, sizeof(buf), "Matroska Plugin v%i.%i",
		(ver >> 24) & 0xFF, (ver >> 16) & 0xFF);

	if (i > 0 && i < sizeof(buf) && pTitle != NULL)
	{
		_snprintf(buf + i, sizeof(buf) - i, " - %s", pTitle);
	}

	buf[sizeof(buf) - 1] = '\0';
	SetWindowTextA(hDlg, buf);
}


INT_PTR CALLBACK InputOptionsMKV::OptionsDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	InputOptionsMKV *thisPtr;
	WORD wChosen;

	switch (message) {

	case WM_INITDIALOG:
		SetWindowLongPtr(hDlg, DWLP_USER, lParam);
		thisPtr = (InputOptionsMKV *)lParam;
		InitDialogTitle(hDlg, "Extended Options");
		CheckDlgButton(hDlg, IDC_CREATE_INDEX, thisPtr->opts.fCreateIndex? BST_CHECKED: BST_UNCHECKED);
		CheckDlgButton(hDlg, IDC_ADD_ADTS_HEADERS, thisPtr->opts.fDeliverRawAAC? BST_UNCHECKED: BST_CHECKED);
		CheckDlgButton(hDlg, IDC_USE_H264_NOT_AVC1, thisPtr->opts.fUseH264NotAVC1? BST_CHECKED: BST_UNCHECKED);
		return TRUE;

	case WM_COMMAND:
		wChosen = LOWORD(wParam);
		if (wChosen == IDC_SAVE || wChosen == IDOK || wChosen == IDCANCEL)
		{
			thisPtr = (InputOptionsMKV *)GetWindowLongPtr(hDlg, DWLP_USER);
			if (wChosen != IDCANCEL)
			{
				thisPtr->opts.fCreateIndex = (IsDlgButtonChecked(hDlg, IDC_CREATE_INDEX) == BST_CHECKED);
				thisPtr->opts.fDeliverRawAAC = (IsDlgButtonChecked(hDlg, IDC_ADD_ADTS_HEADERS) == BST_UNCHECKED);
				thisPtr->opts.fUseH264NotAVC1 = (IsDlgButtonChecked(hDlg, IDC_USE_H264_NOT_AVC1) == BST_CHECKED);
				if (wChosen == IDC_SAVE)
				{
					HKEY hKey;

					if (RegCreateKeyEx(HKEY_CURRENT_USER, MkvRegKey,
						0, NULL, REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS,
						NULL, &hKey, NULL) == ERROR_SUCCESS)
					{
						RegSetValueEx(hKey, "Options", 0, REG_BINARY,
							(CONST BYTE *)&thisPtr->opts, thisPtr->opts.len);

						RegCloseKey(hKey);
					}
				}
			}
			EndDialog(hDlg, TRUE);
			return TRUE;
		}
		break;

	default:
		break;
	}

	return FALSE;
}

void InputOptionsMKV::OptionsDlg(HWND hwndParent)
{
	DialogBoxParam(g_hModule, MAKEINTRESOURCE(IDD_OPTIONS),
		hwndParent, OptionsDlgProc, (LPARAM)this);
}

