#ifndef _MATROSKATRACK_H_
#define _MATROSKATRACK_H_

// Audio Codec IDs
#define A_MPEG_L2      2
#define A_MPEG_L3      3
#define A_MS_ACM       4
#define A_AAC          5
#define A_AC3          6
//#define A_EAC3       7
#define A_FLAC         8
#define A_PCM_INT_LIT  9
#define A_PCM_INT_BIG  10
#define A_VORBIS       11	// Vorbis I, experimental


// Video Codec IDs
#define V_MS_VFW_FOURCC 12
#define V_MPEG4_ISO_AVC 13
#define V_MPEG1         14
#define V_MPEG2         15
#define V_QUICKTIME     16
#define V_REAL_RV10     17
#define V_REAL_RV20     18
#define V_REAL_RV30     19
#define V_REAL_RV40     20
#define V_THEORA        21


// Format tags missing from MMReg.h
#define WAVE_FORMAT_AC3         0x2000
#define WAVE_FORMAT_FLAC        0xF1AC
#define WAVE_FORMAT_MSAUDIO2    0x0161
#define WAVE_FORMAT_MSAUDIO_PRO 0x0162
#define WAVE_FORMAT_RAW_AAC1    0x00FF
#define WAVE_FORMAT_VORBIS_I    0x566F


// Key frame bit (stored in frame size field)
#define MKV_KEYFRAME   0x80000000

// Track types
#define MKV_TYPE_VIDEO 1
#define MKV_TYPE_AUDIO 2

// Maximum supported width and height of a frame
#define MKV_MAX_WIDTH  1920
#define MKV_MAX_HEIGHT 1088

// Flags for the dwFlags member
#define MKV_HAS_VSB    1	// Track has variable size blocks
#define MKV_IS_VBR     2	// Track has a variable bit rate
#define MKV_ADD_ADTS   4	// Add an ADTS header to each AAC frame


class MatroskaTrack {

private:
	DataVector*      dv;

	WORD             wBitsPerSample;
	int              nChannels;

	int              iPixelWidth;
	int              iPixelHeight;

	inline bool ParseContentCompression(MkvFile &mFile, __int64 limit);
	inline bool ParseContentEncoding(MkvFile &mFile, __int64 limit);
	inline bool ParseContentEncodings(MkvFile &mFile, __int64 limit);

	inline bool ParseAudio(MkvFile &mFile, __int64 limit);
	inline bool ParseVideo(MkvFile &mFile, __int64 limit);

    static int  GetCodecID(const char *CodecID);

	bool        ReadStream(MkvFile &mFile, void *lpBuffer, int bytes);

	static int  ac3_sync_info(unsigned long hdr, DWORD *srate, int *brate, int *chans);

	inline bool FormatFLAC();
	inline bool FormatPCM();
	inline bool FormatWMA();
	bool        FormatMPEG(MkvFile &mFile);
	inline bool CreateASC(int sr, int ch);
	inline bool FormatAAC(MkvFile &mFile);
	inline bool FormatAC3(MkvFile &mFile);
	inline bool FormatVorbis();
	inline bool FinishAudio(MkvFile &mFile);

	static int  TransformAVCConfRec(const unsigned char *src, unsigned char *dst);
	inline bool FinishVideo();

	static void FinishStamps(MKVSampleInfo *msi, const DWORD samples);

public:
	MatroskaTrack();
	~MatroskaTrack();

	HIC              hCodec;
	int              iCodecID;
	int              iLanguageID;

	unsigned char*   pFormat;
	int              iFormatLen;

	DWORD            dwSamplingFrequency;
	DWORD            dwOutputSamplingFrequency;

	unsigned char*   pCodecPrivate;
	int              iCodecPrivateLen;
	unsigned char*   pCompSettings;
	int              iCompSettingsLen;
	int              avc_len;

	MKVSampleInfo*   sample_list;
	DWORD            dwActualLength;
	unsigned __int64 ui64TotalBytes;
	DWORD            dwMaxSample;
	double           Duration;

	DWORD            dwRate;
	DWORD            dwScale;
	DWORD            dwFlags;

	int              iTrackNumber;
	int              iTrackType;

	bool    AddBlock(MkvFile &mFile, MKVSampleInfo &msi, int Flags);
	bool    ParseTrackEntry(MkvFile &mFile, __int64 limit);
	bool    Finish(MkvFile &mFile);

	bool    ReadFromIndex(WinFile &idx, DWORD ver);
	bool    WriteToIndex (WinFile &idx) const;
};


#endif  // _MATROSKATRACK_H_

