///////////////////////////////////////////////////////////////////////
// Matroska plugin for VirtualDub 1.8.8+
// Copyright (C) 2010-2012 fccHandler
// Copyright (C) 1998-2012 Avery Lee
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///////////////////////////////////////////////////////////////////////

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include "MkvFile.h"


int MkvFile::ElementID()
{
	int byte = ReadByte();
	int ret  = byte;

	if (byte != 0)
	{
		while (byte < 0x80)
		{
			ret = (ret << 8) + ReadByte();
			byte += byte;
		}
	}

	return ret;
}


__int64 MkvFile::ElementSize()
{
	int     byte = ReadByte();
	__int64 ret  = byte;
	__int64 mask = 0x7F;

	if (byte != 0)
	{
		while (byte < 0x80)
		{
			ret  = (ret  << 8) + ReadByte();
			mask = (mask << 7) + 0x7F;
			byte += byte;
		}
	}

	ret &= mask;
	
	if (ret == mask)
	{
		// Reserved, unknown size
		ret = Size() - Pos();
	}

	return ret;
}


__int64 MkvFile::MatroskaUint()
{
	int     byte = ReadByte();
	__int64 ret  = byte;
	__int64 mask = 0x7F;

	if (byte != 0)
	{
		while (byte < 0x80)
		{
			ret  = (ret  << 8) + ReadByte();
			mask = (mask << 7) + 0x7F;
			byte += byte;
		}
	}

	return (ret & mask);
}


__int64 MkvFile::MatroskaSint()
{
	int     byte  = ReadByte();
	__int64 mask  = 127;
	__int64 scale = 64;
	__int64 ret   = byte & 127;

	if (byte != 0)
	{
		int count = 0;

		while (byte < 0x80)
		{
			++count;
			byte  += byte;
			mask  >>= 1;
			scale <<= 7;
			ret   &= mask;
		}

		while (count > 0)
		{
			ret = (ret << 8) + ReadByte();
			--count;
		}

		ret -= (scale - 1);
	}

	return ret;
}


int MkvFile::MotorolaInt(int size)
{
	int ret = 0;
	while (size > 0)
	{
		ret = (ret << 8) + ReadByte();
		--size;
	}
	return ret;
}


double MkvFile::MotorolaFloat(int size)
{
	unsigned char buf[8];
	int i;

	double ret = 0.0;

	if (size == 4)
	{
		float f;

		for (i = 3; i >= 0; --i)
		{
			buf[i] = ReadByte();
		}

		memcpy(&f, buf, 4);
		ret = f;
	}
	else if (size == 8)
	{
		for (i = 7; i >= 0; --i)
		{
			buf[i] = ReadByte();
		}

		memcpy(&ret, buf, 8);
	}

	return ret;
}


char *MkvFile::MatroskaStringA(int len)
{
	char *ret = NULL;
	if (len > 0)
	{
		ret = new char[len + 1];
		if (ret != NULL)
		{
			Read(ret, len);
			ret[len] = '\0';
		}
	}
	return ret;
}


wchar_t *MkvFile::MatroskaStringW(int len)
{
	// Read a Matroska UTF-8 string and return it as Unicode

	wchar_t *ret = NULL;
	if (len > 0)
	{
		ret = new wchar_t[len + 1];
		if (ret != NULL)
		{
			int ch1, ch2;
			int idx = 0;

			while (len > 0)
			{
				ch1 = ReadByte();
				--len;

				if (ch1 >= 0x80)
				{
					if (len == 0) break;
					ch2 = ReadByte() & 0x3F;
					--len;

					if (ch1 < 0xE0)
					{
						// 110xxxxx 10xxxxxx
						ch1 = ((ch1 & 0x1F) << 6) + ch2;
					}
					else
					{
						if (len == 0) break;
						if (ch1 >= 0xF0) break; // not supported!

						// 1110xxxx 10xxxxxx 10xxxxxx
						ch1 = ((ch1 & 0x0F) << 12) + (ch2 << 6) + (ReadByte() & 0x3F);
						--len;
					}
				}
				
				ret[idx] = (wchar_t)ch1;
				++idx;
			}

			ret[idx] = L'\0';
		}
	}

	return ret;
}

