///////////////////////////////////////////////////////////////////////
// Matroska plugin for VirtualDub 1.8.8+
// Copyright (C) 2010-2012 fccHandler
// Copyright (C) 1998-2012 Avery Lee
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
///////////////////////////////////////////////////////////////////////

// Generic VFW codec support

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <vfw.h>

#include "VDInputDriver.h"
#include "VideoCodec.h"


VideoCodec::VideoCodec()
{
	hCodec       = NULL;
	bmihIn       = NULL;
	mFrameBuffer = NULL;
}


VideoCodec::~VideoCodec()
{ 
	if (hCodec != NULL)
	{
		ICDecompressEnd(hCodec);
		ICClose(hCodec);
	}
	delete[] mFrameBuffer;
	delete[] bmihIn;
}


inline bool VideoCodec::SetInputFormat(const BITMAPINFOHEADER *in)
{
	if (in != NULL)
	{
		int len = in->biSize;

		// Shortcut
		if (bmihIn != NULL)
		{
			if ((int)bmihIn->biSize == len)
			{
				bmihIn->biSizeImage = 0;
				if (memcmp(bmihIn, in, len) == 0) return true;
			}
		}

		if (len >= sizeof(BITMAPINFOHEADER))
		{
			if (in->biWidth  != 0 && in->biWidth  <= 4096 &&
				in->biHeight != 0 && in->biHeight <= 4096)
			{
			
				delete[] bmihIn;
				delete[] mFrameBuffer;

				mFrameBuffer = NULL;
				bmihIn = (BITMAPINFOHEADER *)(new unsigned char[len]);

				if (bmihIn != NULL)
				{
					memcpy(bmihIn, in, len);

					mFrameBuffer = new unsigned char[in->biWidth * in->biHeight * 4];
					if (mFrameBuffer != NULL) return true;

					delete[] bmihIn;
					bmihIn = NULL;
				}
			}
		}
	}

	return false;
}


bool VideoCodec::LockFormat(int format, const OUTPUTFORMAT *out)
{
	int len, w, h;

	len = out->bmih.biSize;
	w   = out->bmih.biWidth;
	h   = out->bmih.biHeight;

	using namespace nsVDXPixmap;

	switch (format) {

	case kPixFormat_XRGB1555:
	case kPixFormat_RGB565:
		memcpy(&bmihOut, out, len);

		mPixmap.pitch   = (w * 2 + 3) & -4;
		mPixmap.data    = mFrameBuffer + (h - 1) * mPixmap.pitch;
		mPixmap.pitch   = -mPixmap.pitch;
		mPixmap.palette = NULL;
		mPixmap.format  = format;
		mPixmap.w       = w;
		mPixmap.h       = h;
		mPixmap.data2   = NULL;
		mPixmap.pitch2  = 0;
		mPixmap.data3   = NULL;
		mPixmap.pitch3  = 0;
		break;

	case kPixFormat_RGB888:
		memcpy(&bmihOut, out, len);

		mPixmap.pitch   = (w * 3 + 3) & -4;
		mPixmap.data    = mFrameBuffer + (h - 1) * mPixmap.pitch;
		mPixmap.pitch   = -mPixmap.pitch;
		mPixmap.palette = NULL;
		mPixmap.format  = format;
		mPixmap.w       = w;
		mPixmap.h       = h;
		mPixmap.data2   = NULL;
		mPixmap.pitch2  = 0;
		mPixmap.data3   = NULL;
		mPixmap.pitch3  = 0;
		break;

	case kPixFormat_XRGB8888:
		memcpy(&bmihOut, out, len);

		mPixmap.pitch   = w * 4;
		mPixmap.data    = mFrameBuffer + (h - 1) * mPixmap.pitch;
		mPixmap.pitch   = -mPixmap.pitch;
		mPixmap.palette = NULL;
		mPixmap.format  = format;
		mPixmap.w       = w;
		mPixmap.h       = h;
		mPixmap.data2   = NULL;
		mPixmap.pitch2  = 0;
		mPixmap.data3   = NULL;
		mPixmap.pitch3  = 0;
		break;

	case kPixFormat_Y8:
		// Fake
		memcpy(&bmihOut, out, len);

		mPixmap.data    = mFrameBuffer;
		mPixmap.pitch   = w;
		mPixmap.palette = NULL;
		mPixmap.format  = format;
		mPixmap.w       = w;
		mPixmap.h       = h;
		mPixmap.data2   = NULL;
		mPixmap.pitch2  = 0;
		mPixmap.data3   = NULL;
		mPixmap.pitch3  = 0;
		break;

	case kPixFormat_YUV422_UYVY:
	case kPixFormat_YUV422_YUYV:
		memcpy(&bmihOut, out, len);

		mPixmap.data    = mFrameBuffer;
		mPixmap.pitch   = w * 2;
		mPixmap.palette = NULL;
		mPixmap.format  = format;
		mPixmap.w       = w;
		mPixmap.h       = h;
		mPixmap.data2   = NULL;
		mPixmap.pitch2  = 0;
		mPixmap.data3   = NULL;
		mPixmap.pitch3  = 0;
		break;

//	case kPixFormat_YUV444_Planar:
//	case kPixFormat_YUV422_Planar:

	case kPixFormat_YUV420_Planar:
		memcpy(&bmihOut, out, len);

		mPixmap.data    = mFrameBuffer;
		mPixmap.pitch   = w;
		mPixmap.palette = NULL;
		mPixmap.format  = format;
		mPixmap.w       = w;
		mPixmap.h       = h;
		mPixmap.data2   = mFrameBuffer + w * h + (w >> 1) * (h >> 1);
		mPixmap.pitch2  = w >> 1;
		mPixmap.data3   = mFrameBuffer + w * h;
		mPixmap.pitch3  = w >> 1;
		break;

//	case kPixFormat_YUV411_Planar:
//	case kPixFormat_YUV410_Planar:

	default:
		return false;
	}

	return true;
}


bool VideoCodec::QueryDecompressor(BITMAPINFOHEADER *out)
{
	bool ret;

	if (hCodec != NULL)
	{
		ret = (ICDecompressQuery(hCodec, bmihIn, out) == ICERR_OK);
	}
	else
	{
		hCodec = ICLocate(ICTYPE_VIDEO, bmihIn->biCompression, bmihIn, out, ICMODE_DECOMPRESS);
		ret = (hCodec != NULL);
	}

	if (ret) ICDecompressBegin(hCodec, bmihIn, out);
	
	return ret;
}


inline bool VideoCodec::QueryOutputFormat(int format)
{
	OUTPUTFORMAT bmihTest;
	int w, h;

	if (bmihIn == NULL) return false;

	w = bmihIn->biWidth;
	h = bmihIn->biHeight;

	using namespace nsVDXPixmap;

	bmihTest.bmih.biSize          = sizeof(BITMAPINFOHEADER);
	bmihTest.bmih.biWidth         = w;
	bmihTest.bmih.biHeight        = h;
	bmihTest.bmih.biSizeImage     = 0;
	bmihTest.bmih.biXPelsPerMeter = 0;
	bmihTest.bmih.biYPelsPerMeter = 0;
	bmihTest.bmih.biClrUsed       = 0;
	bmihTest.bmih.biClrImportant  = 0;
	bmihTest.bmiColors[0]         = 0;
	bmihTest.bmiColors[1]         = 0;
	bmihTest.bmiColors[2]         = 0;

	switch (format) {

	case kPixFormat_Null:

		if ((w & 1) == 0)
		{
			if ((h & 1) == 0)
			{
				// Try 4:2:0 planar

				bmihTest.bmih.biPlanes      = 3;
				bmihTest.bmih.biBitCount    = 12;
				bmihTest.bmih.biCompression = '21VY';
				bmihTest.bmih.biSizeImage   = (w * h) + (w >> 1) * (h >> 1) * 2;

				if (QueryDecompressor(&(bmihTest.bmih)))
				{
					return LockFormat(kPixFormat_YUV420_Planar, &bmihTest);
				}
			}

			// Try YUY2

			bmihTest.bmih.biPlanes      = 1;
			bmihTest.bmih.biBitCount    = 16;
			bmihTest.bmih.biCompression = '2YUY';
			bmihTest.bmih.biSizeImage   = ((w * 2 + 3) & -4) * h;

			if (QueryDecompressor(&(bmihTest.bmih)))
			{
				return LockFormat(kPixFormat_YUV422_YUYV, &bmihTest);
			}
		}

		// Try RGB32
		bmihTest.bmih.biPlanes      = 1;
		bmihTest.bmih.biBitCount    = 32;
		bmihTest.bmih.biCompression = BI_RGB;

		if (QueryDecompressor(&(bmihTest.bmih)))
		{
			return LockFormat(kPixFormat_XRGB8888, &bmihTest);
		}
		// Fall through...

	case kPixFormat_RGB888:
		bmihTest.bmih.biPlanes      = 1;
		bmihTest.bmih.biBitCount    = 24;
		bmihTest.bmih.biCompression = BI_RGB;

		if (!QueryDecompressor(&(bmihTest.bmih))) return false;
		return LockFormat(kPixFormat_RGB888, &bmihTest);

	case kPixFormat_XRGB8888:
		bmihTest.bmih.biPlanes      = 1;
		bmihTest.bmih.biBitCount    = 32;
		bmihTest.bmih.biCompression = BI_RGB;

		if (!QueryDecompressor(&(bmihTest.bmih))) return false;
		return LockFormat(kPixFormat_XRGB8888, &bmihTest);

	case kPixFormat_XRGB1555:
		bmihTest.bmih.biPlanes      = 1;
		bmihTest.bmih.biBitCount    = 16;
		bmihTest.bmih.biCompression = BI_RGB;

		if (!QueryDecompressor(&(bmihTest.bmih))) return false;
		return LockFormat(kPixFormat_XRGB1555, &bmihTest);

	case kPixFormat_RGB565:
		bmihTest.bmih.biSize        = sizeof(OUTPUTFORMAT);
		bmihTest.bmih.biPlanes      = 1;
		bmihTest.bmih.biBitCount    = 16;
		bmihTest.bmih.biCompression = BI_BITFIELDS;
		bmihTest.bmiColors[0]       = 0xF800;
		bmihTest.bmiColors[1]       = 0x07E0;
		bmihTest.bmiColors[2]       = 0x001F;

		if (!QueryDecompressor(&(bmihTest.bmih))) return false;
		bmihTest.bmih.biPlanes = 1;
		return LockFormat(kPixFormat_RGB565, &bmihTest);

	case kPixFormat_Y8:
		// Fake this if possible
		if ((w | h) & 1) return false;
		bmihTest.bmih.biPlanes      = 3;
		bmihTest.bmih.biBitCount    = 12;
		bmihTest.bmih.biCompression = '21VY';
		bmihTest.bmih.biSizeImage   = (w * h) + (w >> 1) * (h >> 1) * 2;

		if (!QueryDecompressor(&(bmihTest.bmih))) return false;
		return LockFormat(kPixFormat_Y8, &bmihTest);

	case kPixFormat_YUV422_UYVY:
		if (w & 1) return false;
		bmihTest.bmih.biPlanes      = 1;
		bmihTest.bmih.biBitCount    = 16;
		bmihTest.bmih.biCompression = 'YVYU';
		bmihTest.bmih.biSizeImage   = (w * 2) * h;

		if (!QueryDecompressor(&(bmihTest.bmih))) return false;
		return LockFormat(kPixFormat_YUV422_UYVY, &bmihTest);

	case kPixFormat_YUV422_YUYV:
		if (w & 1) return false;
		bmihTest.bmih.biPlanes      = 1;
		bmihTest.bmih.biBitCount    = 16;
		bmihTest.bmih.biCompression = '2YUY';
		bmihTest.bmih.biSizeImage   = (w * 2) * h;

		if (!QueryDecompressor(&(bmihTest.bmih))) return false;
		return LockFormat(kPixFormat_YUV422_YUYV, &bmihTest);

//	case kPixFormat_YUV444_Planar:
//	case kPixFormat_YUV422_Planar:

	case kPixFormat_YUV420_Planar:
		if ((w | h) & 1) return false;

		bmihTest.bmih.biPlanes      = 3;
		bmihTest.bmih.biBitCount    = 12;
		bmihTest.bmih.biCompression = '21VY';
		bmihTest.bmih.biSizeImage   = (w * h) + (w >> 1) * (h >> 1) * 2;

		if (!QueryDecompressor(&(bmihTest.bmih))) return false;
		return LockFormat(kPixFormat_YUV420_Planar, &bmihTest);

//	case kPixFormat_YUV411_Planar:
//	case kPixFormat_YUV410_Planar:

	default:
		return false;
	}

	return true;
}


bool VideoCodec::Init(const BITMAPINFOHEADER *in, int format)
{
	if (in == NULL)
	{
		// Emulated
		if (hCodec != NULL)
		{
			ICDecompressEnd(hCodec);
			ICClose(hCodec);
			hCodec = NULL;
		}
		
		delete[] bmihIn;
		bmihIn = NULL;
		
		delete[] mFrameBuffer;
		mFrameBuffer = NULL;

		if (   format == nsVDXPixmap::kPixFormat_XRGB8888
		    || format == nsVDXPixmap::kPixFormat_Null)
		{

			mFrameBuffer = new unsigned char[320 * 240 * 4];

			if (mFrameBuffer != NULL)
			{
				memset(mFrameBuffer, 0, 320 * 240 * 4);

				bmihOut.bmih.biSize          = sizeof(BITMAPINFOHEADER);
				bmihOut.bmih.biWidth         = 320;
				bmihOut.bmih.biHeight        = 240;
				bmihOut.bmih.biPlanes        = 1;
				bmihOut.bmih.biBitCount      = 32;
				bmihOut.bmih.biCompression   = BI_RGB;
				bmihOut.bmih.biSizeImage     = 0;
				bmihOut.bmih.biXPelsPerMeter = 0;
				bmihOut.bmih.biYPelsPerMeter = 0;
				bmihOut.bmih.biClrUsed       = 0;
				bmihOut.bmih.biClrImportant  = 0;

				mPixmap.data    = mFrameBuffer + 239 * 320 * 4;
				mPixmap.pitch   = 320 * -4;
				mPixmap.palette = NULL;
				mPixmap.format  = nsVDXPixmap::kPixFormat_XRGB8888;
				mPixmap.w       = 320;
				mPixmap.h       = 240;
				mPixmap.data2   = NULL;
				mPixmap.pitch2  = 0;
				mPixmap.data3   = NULL;
				mPixmap.pitch3  = 0;

				return true;
			}
		}
		return false;
	}

	if (SetInputFormat(in))
	{
		// Even if we fail to obtain a codec we must
		// return something, or VirtualDub will crash!

		memset(mFrameBuffer, 0, in->biWidth * in->biHeight * 4);

		bmihOut.bmih.biSize          = sizeof(BITMAPINFOHEADER);
		bmihOut.bmih.biWidth         = in->biWidth;
		bmihOut.bmih.biHeight        = in->biHeight;
		bmihOut.bmih.biPlanes        = 1;
		bmihOut.bmih.biBitCount      = 32;
		bmihOut.bmih.biCompression   = BI_RGB;
		bmihOut.bmih.biSizeImage     = 0;
		bmihOut.bmih.biXPelsPerMeter = 0;
		bmihOut.bmih.biYPelsPerMeter = 0;
		bmihOut.bmih.biClrUsed       = 0;
		bmihOut.bmih.biClrImportant  = 0;

		mPixmap.data    = mFrameBuffer + (in->biHeight - 1) * in->biWidth * 4;
		mPixmap.pitch   = in->biWidth * -4;
		mPixmap.palette = NULL;
		mPixmap.format  = nsVDXPixmap::kPixFormat_XRGB8888;
		mPixmap.w       = in->biWidth;
		mPixmap.h       = in->biHeight;
		mPixmap.data2   = NULL;
		mPixmap.pitch2  = 0;
		mPixmap.data3   = NULL;
		mPixmap.pitch3  = 0;

		return QueryOutputFormat(format);
	}

	return false;
}


bool VideoCodec::Decompress(LPVOID inputBuffer, DWORD len, DWORD flags)
{
	if (hCodec != NULL && bmihIn != NULL && inputBuffer != NULL && len > 0)
	{
		DWORD ret;

		bmihIn->biSizeImage = len;

		ret = ICDecompress(hCodec, flags, bmihIn, inputBuffer,
		    &(bmihOut.bmih), mFrameBuffer);

		return (ret == ICERR_OK);
	}

	return false;
}

void VideoCodec::Inform(LPCSTR szText)
{
	HBITMAP hOldBmp;
	HBITMAP hNewBmp;
	int     len;
	HDC     hdc;
	RECT    rc;

	hdc     = NULL;
	hOldBmp = NULL;
	hNewBmp = NULL;

	if (szText == NULL) goto Abort;

	len = lstrlenA(szText);
	if (len <= 0) goto Abort;

	hdc = CreateCompatibleDC(NULL);
	if (hdc == NULL) goto Abort;

	hNewBmp = CreateCompatibleBitmap(hdc, bmihOut.bmih.biWidth, bmihOut.bmih.biHeight);
	if (hNewBmp == NULL) goto Abort;

	hOldBmp = (HBITMAP)SelectObject(hdc, hNewBmp);
	if (hOldBmp == NULL) goto Abort;

	rc.left   = 0;
	rc.right  = bmihOut.bmih.biWidth;
	rc.top    = 0;
	rc.bottom = bmihOut.bmih.biHeight;

	FillRect(hdc, &rc, (HBRUSH)GetStockObject(BLACK_BRUSH));

	SetBkColor  (hdc, (COLORREF)0);
	SetTextColor(hdc, (COLORREF)0x00FFFFFF);
	SetTextAlign(hdc, TA_CENTER);

	ExtTextOutA(
		hdc,
		bmihOut.bmih.biWidth >> 1,
		bmihOut.bmih.biHeight >> 1,
		ETO_CLIPPED,
		&rc,
		szText,
		len,
		NULL
	);

	// Now copy the rendered text...
	// First, we must de-select the bitmap

	SelectObject(hdc, hOldBmp);
	hOldBmp = NULL;

	GetDIBits(hdc, hNewBmp, 0, bmihOut.bmih.biHeight,
		mFrameBuffer, (BITMAPINFO *)&bmihOut, DIB_RGB_COLORS);

Abort:
	if (hdc != NULL)
	{
		if (hNewBmp != NULL)
		{
			if (hOldBmp != NULL) SelectObject(hdc, hOldBmp);
			DeleteObject(hNewBmp);
		}
		DeleteDC(hdc);
	}
}

