#ifndef _WINFILE_H_
#define _WINFILE_H_

class WinFile {

private:
	enum {
		WF_WRITE        = 1,
		WF_DIRTY        = 2,
		WF_CLOSE        = 4,
		WF_BUFFERSIZE   = 65536
	};

	unsigned char   m_Buff[WF_BUFFERSIZE];
	LARGE_INTEGER   m_BuffPos;
	LARGE_INTEGER   m_Pos;
	LARGE_INTEGER   m_Size;
	HANDLE          m_Handle;
	int             m_Flags;
	int             m_CurByte;
	int             m_CurBit;

	bool Flush();
	bool Refresh();
	bool Kill();

	HANDLE DualCreateFile(
		LPCWSTR                 lpFileName,
		DWORD                   dwDesiredAccess,
		DWORD                   dwShareMode,
		LPSECURITY_ATTRIBUTES   lpSecurityAttributes,
		DWORD                   dwCreationDisposition,
		DWORD                   dwFlagsAndAttributes,
		HANDLE                  hTemplateFile
	) const;

public:
	WinFile::WinFile();
	WinFile::~WinFile();

	// Convert a wide character string to ANSI
	static LPSTR WcharToAnsi(LPCWSTR lpszWchar);

	// Delete a file, ANSI version
	static bool Delete(const char *lpszFileName);

	// Delete a file, wide character version
	static bool Delete(const wchar_t *lpszFileName);

	// Bind to an already-open file handle
	bool Bind(HANDLE h, bool bWriteAccess = false);

	// Open a new file, possibly for writing
	bool Open(const char *lpszFileName, bool bWriteAccess = false);

	// Open a new file, possibly for writing
	bool Open(const wchar_t *lpszFileName, bool bWriteAccess = false);

	// Close this file
	bool Close();

	// Return the current size of this file
	__int64 Size() const { return m_Size.QuadPart; }

	// Return the current position in this file
	__int64 Pos() const { return m_Pos.QuadPart; }

	// Return the file handle
	HANDLE Handle() const { return m_Handle; }

	// Return whether we are at the end of the file
	bool EndOfFile() const { return (m_Pos.QuadPart >= m_Size.QuadPart); }

	// Seek to an absolute position
	bool Seek(__int64 iAbsPos);

	// Move the file pointer a relative distance
	bool Move(int iDistance);

	// Read from the file and store in buffer
	int Read(void *lpBuffer, int iBytes);

	// Write the buffer to the file
	bool Write(const void *lpBuffer, int iBytes);

	// Read one byte from the file
	int ReadByte();

	// Write one byte to the file
	bool WriteByte(int iByte);

	// Align at the next byte boundary
	void AlignToByte();

	// Read one bit from the file
	int GetBit();

	// Read multiple bits from the file
	int GetBits(int nBits);
};

#endif	// _WINFILE_H_
